@@ -1,5 +1,6 @@
 package org.postgresql.util;
 
+import org.postgresql.Driver;
 import java.io.*;
 import java.lang.*;
 import java.lang.reflect.*;
@@ -56,7 +57,7 @@ import java.sql.*;
  * There are a number of limitations placed on the java class to be
  * used by Serialize:
  * <ul>
- * <li>The class name must be less than 32 chars long and must be all lowercase.
+ * <li>The class name must be less than 64 chars long and must be all lowercase.
  * This is due to limitations in Postgres about the size of table names.
  * The name must be all lowercase since table names in Postgres are
  * case insensitive and the relname is stored in lowercase. Unless some
@@ -108,7 +109,7 @@ import java.sql.*;
 public class Serialize
 {
 	// This is the connection that the instance refers to
-	protected org.postgresql.Connection conn;
+	protected Connection conn;
 
 	// This is the table name
 	protected String tableName;
@@ -123,38 +124,49 @@ public class Serialize
 	 * This creates an instance that can be used to serialize or deserialize
 	 * a Java object from a PostgreSQL table.
 	 */
-	public Serialize(org.postgresql.Connection c, String type) throws SQLException
+	public Serialize(Connection conn, String type) throws SQLException
 	{
 		try
 		{
-			conn = c;
-			DriverManager.println("Serialize: initializing instance for type: " + type);
-			tableName = toPostgreSQL(type);
+			this.conn = conn;
+			if (Driver.logDebug)
+				Driver.debug("Serialize: initializing instance for type: " + type);
+			tableName = toPostgreSQL(conn,type);
 			className = type;
 			ourClass = Class.forName(className);
 		}
 		catch (ClassNotFoundException cnfe)
 		{
-			DriverManager.println("Serialize: " + className + " java class not found");
+			if (Driver.logDebug)
+				Driver.debug("Serialize: " + className + " java class not found");
 			throw new PSQLException("postgresql.serial.noclass", type);
 		}
 
 		// Second check, the type must be a table
 		boolean status = false;
-		ResultSet rs = conn.ExecSQL("select typname from pg_type,pg_class where typname=relname and typname='" + tableName + "'");
+		String sql;
+		if (conn.getMetaData().supportsSchemasInTableDefinitions()) {
+			sql = "SELECT 1 FROM pg_catalog.pg_type t, pg_catalog.pg_class c WHERE t.typrelid=c.oid AND c.relkind='r' AND t.typname='" + tableName + "' AND pg_table_is_visible(c.oid) ";
+		} else {
+			sql = "SELECT 1 FROM pg_type t, pg_class c WHERE t.typrelid=c.oid AND c.relkind='r' AND t.typname='"+tableName+"'";
+		}
+
+		ResultSet rs = conn.createStatement().executeQuery(sql);
 		if (rs != null)
 		{
 			if (rs.next())
 			{
 				status = true;
-				DriverManager.println("Serialize: " + tableName + " table found");
+				if (Driver.logDebug)
+					Driver.debug("Serialize: " + tableName + " table found");
 			}
 			rs.close();
 		}
 		// This should never occur, as org.postgresql has it's own internal checks
 		if (!status)
 		{
-			DriverManager.println("Serialize: " + tableName + " table not found");
+			if (Driver.logDebug)
+				Driver.debug("Serialize: " + tableName + " table not found");
 			throw new PSQLException("postgresql.serial.table", type);
 		}
 		// Finally cache the fields within the table
@@ -163,7 +175,7 @@ public class Serialize
 	/*
 	 * Constructor when Object is passed in
 	 */
-	public Serialize(org.postgresql.Connection c, Object o) throws SQLException
+	public Serialize(Connection c, Object o) throws SQLException
 	{
 		this(c, o.getClass().getName());
 	}
@@ -171,7 +183,7 @@ public class Serialize
 	/*
 	 * Constructor when Class is passed in
 	 */
-	public Serialize(org.postgresql.Connection c, Class cls) throws SQLException
+	public Serialize(Connection c, Class cls) throws SQLException
 	{
 		this(c, cls.getName());
 	}
@@ -182,13 +194,15 @@ public class Serialize
 	 * @return Object relating to oid
 	 * @exception SQLException on error
 	 */
-	public Object fetch(int oid) throws SQLException
+	public Object fetch(long oid) throws SQLException
 	{
 		try
 		{
-			DriverManager.println("Serialize.fetch: " + "attempting to instantiate object of type: " + ourClass.getName() );
+			if (Driver.logDebug)
+				Driver.debug("Serialize.fetch: " + "attempting to instantiate object of type: " + ourClass.getName() );
 			Object obj = ourClass.newInstance();
-			DriverManager.println("Serialize.fetch: " + "instantiated object of type: " + ourClass.getName() );
+			if (Driver.logDebug)
+				Driver.debug("Serialize.fetch: " + "instantiated object of type: " + ourClass.getName() );
 
 			// NB: we use java.lang.reflect here to prevent confusion with
 			// the org.postgresql.Field
@@ -219,8 +233,9 @@ public class Serialize
 			sb.append(" where oid=");
 			sb.append(oid);
 
-			DriverManager.println("Serialize.fetch: " + sb.toString());
-			ResultSet rs = conn.ExecSQL(sb.toString());
+			if (Driver.logDebug)
+				Driver.debug("Serialize.fetch: " + sb.toString());
+			ResultSet rs = conn.createStatement().executeQuery(sb.toString());
 
 			if (rs != null)
 			{
@@ -269,13 +284,13 @@ public class Serialize
 
 	/*
 	 * This stores an object into a table, returning it's OID.<p>
-         * This method was deprecated in 7.2 because the value of an OID
-         * can be larger than a java signed int.
+			* This method was deprecated in 7.2 because the value of an OID
+			* can be larger than a java signed int.
 	 * @deprecated Replaced by storeObject() in 7.2
 	 */
 	public int store(Object o) throws SQLException
 	{
-	    return (int) storeObject(o);
+		return (int) storeObject(o);
 	}
 
 	/*
@@ -295,7 +310,7 @@ public class Serialize
 	 * @param o Object to store (must implement Serializable)
 	 * @return oid of stored object
 	 * @exception SQLException on error
-         * @since 7.2
+			* @since 7.2
 	 */
 	public long storeObject(Object o) throws SQLException
 	{
@@ -388,8 +403,9 @@ public class Serialize
 				sb.append(')');
 			}
 
-			DriverManager.println("Serialize.store: " + sb.toString() );
-			org.postgresql.ResultSet rs = (org.postgresql.ResultSet) conn.ExecSQL(sb.toString());
+			if (Driver.logDebug)
+				Driver.debug("Serialize.store: " + sb.toString() );
+			ResultSet rs = ((org.postgresql.jdbc1.AbstractJdbc1Connection)conn).ExecSQL(sb.toString());
 
 			// fetch the OID for returning
 			if (update)
@@ -402,7 +418,7 @@ public class Serialize
 			else
 			{
 				// new record inserted has new oid; rs should be not null
-				long newOID = ((org.postgresql.ResultSet)rs).getLastOID();
+				long newOID = ((org.postgresql.jdbc1.AbstractJdbc1ResultSet)rs).getLastOID();
 				rs.close();
 				// update the java object's oid field if it has the oid field
 				if (hasOID)
@@ -471,7 +487,7 @@ public class Serialize
 	 * @param o Object to base table on
 	 * @exception SQLException on error
 	 */
-	public static void create(org.postgresql.Connection con, Object o) throws SQLException
+	public static void create(Connection con, Object o) throws SQLException
 	{
 		create(con, o.getClass());
 	}
@@ -484,24 +500,33 @@ public class Serialize
 	 * @param o Class to base table on
 	 * @exception SQLException on error
 	 */
-	public static void create(org.postgresql.Connection con, Class c) throws SQLException
+	public static void create(Connection conn, Class c) throws SQLException
 	{
 		if (c.isInterface())
 			throw new PSQLException("postgresql.serial.interface");
 
 		// See if the table exists
-		String tableName = toPostgreSQL(c.getName());
+		String tableName = toPostgreSQL(conn,c.getName());
 
-		ResultSet rs = con.ExecSQL("select relname from pg_class where relname = '" + tableName + "'");
+		String sql;
+		if (conn.getMetaData().supportsSchemasInTableDefinitions()) {
+			sql = "SELECT 1 FROM pg_catalog.pg_class WHERE relkind='r' AND relname='" + tableName + "' AND pg_table_is_visible(oid) ";
+		} else {
+			sql = "SELECT 1 FROM pg_class WHERE relkind='r' AND relname='"+tableName+"'";
+		}
+
+		ResultSet rs = conn.createStatement().executeQuery(sql);
 		if ( rs.next() )
 		{
-			DriverManager.println("Serialize.create: table " + tableName + " exists, skipping");
+			if (Driver.logDebug)
+				Driver.debug("Serialize.create: table " + tableName + " exists, skipping");
 			rs.close();
-			return;
+			return ;
 		}
 
 		// else table not found, so create it
-		DriverManager.println("Serialize.create: table " + tableName + " not found, creating" );
+		if (Driver.logDebug)
+			Driver.debug("Serialize.create: table " + tableName + " not found, creating" );
 		// No entries returned, so the table doesn't exist
 
 		StringBuffer sb = new StringBuffer("create table ");
@@ -538,8 +563,8 @@ public class Serialize
 						sb.append(tp[j][1]);
 					else
 					{
-						create(con, type);
-						sb.append(toPostgreSQL(n));
+						create(conn, type);
+						sb.append(toPostgreSQL(conn,n));
 					}
 				}
 			}
@@ -547,8 +572,9 @@ public class Serialize
 		sb.append(")");
 
 		// Now create the table
-		DriverManager.println("Serialize.create: " + sb );
-		con.ExecSQL(sb.toString());
+		if (Driver.logDebug)
+			Driver.debug("Serialize.create: " + sb );
+		conn.createStatement().executeUpdate(sb.toString());
 	}
 
 	// This is used to translate between Java primitives and PostgreSQL types.
@@ -570,35 +596,53 @@ public class Serialize
 											 {"byte", "int2"}
 										 };
 
-	/*
+	/**
 	 * This converts a Java Class name to a org.postgresql table, by replacing . with
 	 * _<p>
 	 *
 	 * Because of this, a Class name may not have _ in the name.<p>
 	 * Another limitation, is that the entire class name (including packages)
-	 * cannot be longer than 32 characters (a limit forced by PostgreSQL).
+	 * cannot be longer than the maximum table name length.
 	 *
+	 * @param con The database connection
 	 * @param name Class name
 	 * @return PostgreSQL table name
 	 * @exception SQLException on error
+	 * @since 7.3
 	 */
-	public static String toPostgreSQL(String name) throws SQLException
+	public static String toPostgreSQL(Connection con, String name) throws SQLException
 	{
+		DatabaseMetaData dbmd = con.getMetaData();
+		int maxNameLength = dbmd.getMaxTableNameLength();
+		return toPostgreSQL(maxNameLength,name);
+	}
+
+	/**
+	 * Convert a Java Class Name to an org.postgresql table name, by replacing .
+	 * with _ <p>
+	 *
+	 * @deprecated Replaced by toPostgresql(connection, name) in 7.3
+	 */
+	public static String toPostgreSQL(String name) throws SQLException {
+		return toPostgreSQL(31,name);
+	}
+
+	private static String toPostgreSQL(int maxNameLength, String name) throws SQLException {
+
 		name = name.toLowerCase();
 
 		if (name.indexOf("_") > -1)
 			throw new PSQLException("postgresql.serial.underscore");
 
-		// Postgres table names can only be 32 character long.
-		// Reserve 1 char, so allow only up to 31 chars.
+		// Postgres table names can only be so many characters long.
 		// If the full class name with package is too long
 		// then just use the class name. If the class name is
 		// too long throw an exception.
 		//
-		if ( name.length() > 31 )
+		if ( name.length() > maxNameLength )
 		{
 			name = name.substring(name.lastIndexOf(".") + 1);
-			if ( name.length() > 31 )
+			if ( name.length() > maxNameLength )
 				throw new PSQLException("postgresql.serial.namelength", name, new Integer(name.length()));
 		}
 		return name.replace('.', '_');
